#  Copyright 2023 Nyria
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <https://www.gnu.org/licenses/>.

from typing import Union

from ory.components.pod import Pod
from ory.ext.exceptions import PodAlreadyExistsException, PriorityAlreadyExistsException, PermissionException
from ory.states.permission import Permission


class Pool:
    """
    A pool is a collection of pods that are registered to a specific conditions.
    """

    def __init__(self, name: str, permission: Permission, priority: int = 0) -> None:
        self.__name = name.lower()
        self.__permission = permission
        self.__priority = priority

        self.__pods = dict()

    def get_name(self) -> str:
        """
        Gets the name of the pool
        :return: name of the pool
        """

        return self.__name

    def get_permission(self) -> Permission:
        """
        Gets the permission of the pool
        :return: permission of the pool
        """

        return self.__permission

    def get_priority(self) -> int:
        """
        Gets the priority of the pool
        :return: priority of the pool
        """

        return self.__priority

    def register_pod(self, pod: Pod, override: bool = False) -> None:
        """
        Registers a pod to the pool
        :param pod: The pod to register
        :param override: If the pod should override an existing pod
        :return: None
        """

        if pod.get_name() in self.__pods and not override:
            raise PodAlreadyExistsException("Pod already registered")

        for registered_pod in self.__pods.values():
            if pod.get_priority() == registered_pod.get_priority() and pod.get_priority() != 0:
                raise PriorityAlreadyExistsException("Priority already exists")

            if pod.get_name() == registered_pod.get_name():
                raise PodAlreadyExistsException("Name already registered")

        if override and self.__permission == Permission.READ_ONLY:
            raise PermissionException("This pool is read only")

        self.__pods[pod.get_name()] = pod

    def register_pods(self, pods: list[Pod], override: bool = False) -> None:
        """
        Registers multiple pods to the pool
        :param pods: The pods to register
        :param override: If the pods should override an existing pod
        :return: None
        """

        for pod in pods:
            self.register_pod(pod, override)

    def unregister_pod(self, pod: Pod) -> None:
        """
        Unregisters a pod from the pool
        :param pod: The pod to unregister
        :return: None
        """

        if pod.get_name() not in self.__pods:
            raise PodAlreadyExistsException("Pod already registered")

        if self.__permission == Permission.READ_ONLY:
            raise PermissionException("This pool is read only")

        del self.__pods[pod.get_name()]

    def unregister_pods(self, pods: list[Pod]) -> None:
        """
        Unregisters multiple pods from the pool
        :param pods: The pods to unregister
        :return: None
        """

        for pod in pods:
            self.unregister_pod(pod)

    def get_all_pods(self) -> list:
        """
        Gets the pods of the pool
        :return: pods of the pool
        """

        return list(self.__pods.values())

    def get_pod_by_name(self, name: str) -> Union[Pod, None]:
        """
        Gets a pod by its name.
        :param name: name of the pod.
        :return: Union[Pod, None]
        """

        for pod in self.__pods.values():
            if pod.get_name() == name.lower():
                return pod

        return None

    def get_pod_by_priority(self, priority: int) -> Union[list[Pod], Pod, None]:
        """
        Gets a pod by its priority.
        :param priority: priority of the pod.
        :return: pod with the given priority.
        """

        if priority == 0:
            return [pod for pod in self.__pods.values() if pod.get_priority() == 0]

        for pod in self.__pods.values():
            if pod.get_priority() == priority:
                return pod

        return None

    def get_standard_pods(self) -> list[Pod]:
        """
        Gets all pods that are not prioritized.
        :return: list[Pod]
        """

        return [pod for pod in self.__pods.values() if pod.get_priority() == 0]



#  Copyright 2023 Nyria
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <https://www.gnu.org/licenses/>.

from typing import Union
from ory.states.permission import Permission

from ory.components.pool import Pool
from ory.ext.exceptions import PoolAlreadyExistsException, PriorityAlreadyExistsException
from ory.registry.pool_registry import PoolRegistry


class RegistryHandler:
    """
    This class is used to communicate with the pool registry. It handels all requests.
    """

    @staticmethod
    def create_new_pool(pool: Pool, override: bool = False) -> None:
        """
        Create new service pool

        Attributes:
        ------------
        :param pool: The pool to create
        :param override: If the pool should be overriden if it already exists
        :return: None
        ------------
        """

        if pool.get_name() in PoolRegistry.pool_storage and not override:
            raise PoolAlreadyExistsException("Service pool already exists")

        for registered_pool in PoolRegistry.pool_storage.values():
            if registered_pool.get_priority() == pool.get_priority() and pool.get_priority() != 0:
                raise PriorityAlreadyExistsException("Service pool with same priority already exists")

        if pool.get_name() in PoolRegistry.pool_storage and not override:
            raise PoolAlreadyExistsException("Service pool already exists")

        PoolRegistry.pool_storage[pool.get_name()] = pool

    def create_new_pools(self, pools: list[Pool], override: bool = False) -> None:
        """
        Create multiple new service pools
        :param pools: The pools to create
        :param override: If the pools should be overriden if they already exists
        :return:
        """

        for pool in pools:
            self.create_new_pool(pool, override)

    @staticmethod
    def delete_pool(pool: Pool) -> None:
        """
        Delete a pool by its name

        Attributes:
        ------------
        :param pool: The pool to delete
        :return: None
        ------------
        """

        if pool.get_permission() != Permission.ALL:
            raise PermissionError("Missing permission to delete pool")

        del PoolRegistry.pool_storage[pool.get_name()]

    @staticmethod
    def get_all_pools() -> list:
        """
        Get all pools

        Attributes:
        ------------
        :return: All pools
        ------------
        """

        return [pool for pool in PoolRegistry.pool_storage.values()]

    @staticmethod
    def get_pool_by_name(name: str) -> Union[Pool, None]:
        """
        Get a pool by its name

        Attributes:
        ------------
        :param name: The name of the pool.
        :return: The pool
        ------------
        """

        for pool in PoolRegistry.pool_storage.values():
            if pool.get_name() == name.lower():
                return pool

        return None

    @staticmethod
    def get_pool_by_priority(priority: int) -> Union[list[Pool], Pool, None]:
        """
        Get a pool by its priority

        Attributes:
        ------------
        :param priority: The priority of the pool.
        :return: The pool
        ------------
        """

        if priority == 0:
            return [pool for pool in PoolRegistry.pool_storage.values() if pool.get_priority() == 0]

        for pool in PoolRegistry.pool_storage.values():
            if pool.get_priority() == priority:
                return pool

        return None

    @staticmethod
    def get_standard_pools() -> list[Pool]:
        """
        Get all standard pools

        Attributes:
        ------------
        :return: All standard pools
        ------------
        """

        return [pool for pool in PoolRegistry.pool_storage.values() if pool.get_priority() == 0]



#  Copyright 2023 Nyria
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <https://www.gnu.org/licenses/>.

class PermissionException(Exception):
    """
    This error is raised when the programm tries to change states it is not allowed to do
    """

    def __init__(self, message: str):
        super().__init__(message)


class PodAlreadyExistsException(Exception):
    """
    This error is raised when the programm tries to register a pod that already exists
    """

    def __init__(self, message: str):
        super().__init__(message)


class PriorityAlreadyExistsException(Exception):
    """
    This error is raised when the programm tries to register a pod that have the same priority
    """

    def __init__(self, message: str):
        super().__init__(message)


class PoolAlreadyExistsException(Exception):
    """
    This error is raised when the programm tries to register a pool that already exists
    """

    def __init__(self, message: str):
        super().__init__(message)



#  Copyright 2023 Nyria
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <https://www.gnu.org/licenses/>.

import os

from pathlib import Path
from setuptools import setup, find_packages

# Gets the root path of the project
ROOT = Path(__file__).parent.absolute()

NAME = "nyria-ory"
VERSION = os.environ["CI_COMMIT_TAG"]
DESCRIPTION = "Service Management System"
LONG_DESCRIPTION = "Service Management System"
AUTHOR = "Nyria"
URL = "https://gitlab.nyria.net/nyria/libaries/ory"
LICENSE = "GPL-3.0"

# Get the requirements from the requirements.txt file
with open("requirements.txt", "r") as f:
    install_requires = f.read().splitlines()

# Run the setup
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author=AUTHOR,
    url=URL,
    license=LICENSE,
    packages=find_packages(),
    install_requires=install_requires,
    long_description_content_type="text/markdown"
)
